package data.missions.dara_race02;

import com.fs.starfarer.api.combat.BattleCreationContext;
import com.fs.starfarer.api.fleet.FleetGoal;
import com.fs.starfarer.api.fleet.FleetMemberType;
import com.fs.starfarer.api.impl.combat.EscapeRevealPlugin;
import com.fs.starfarer.api.mission.FleetSide;
import com.fs.starfarer.api.mission.MissionDefinitionAPI;
import com.fs.starfarer.api.mission.MissionDefinitionPlugin;

import java.util.ArrayList;
import java.util.List;

import com.fs.starfarer.api.impl.campaign.ids.Personalities;
//import com.fs.starfarer.api.campaign.CargoAPI.CrewXPLevel;

public class MissionDefinition implements MissionDefinitionPlugin {

	private List ships = new ArrayList();
        private final int numOpponents = 5;
    
    private void generateFleet(FleetSide side, List ships, MissionDefinitionAPI api) {

            String [] choices = {
		"dara_arrow_racer_131_Hull",
		"dara_arrow_racer_yellow_Hull",
		"dara_arrow_racer_red_Hull",
		"dara_arrow_racer_blue_Hull",
                "dara_arrow_racer_aqua_Hull",           // Contributed by Vayra
                "dara_arrow_racer_black_Hull",
                "dara_arrow_racer_bluechecker_Hull",    // Contributed by HELMUT
                "dara_arrow_racer_green_Hull",          // Contributed by HELMUT
                "dara_arrow_racer_ice_Hull",            // Contributed by HELMUT
                "dara_arrow_racer_marshmallow_Hull",    // Contributed by HELMUT
                "dara_arrow_racer_redstar_Hull",        // Contributed by HELMUT
                "dara_arrow_racer_spider_Hull",         // Contributed by HELMUT
                "dara_arrow_racer_fairy_Hull"           // Contributed by Nia
            };
            List<String> choiceList = new ArrayList<>(); // Initiate empty list
            for (String this_choice : choices) {
                choiceList.add(this_choice);
            }
            
            int selection = (int) (Math.random() * (float) choiceList.size());
            String flagship = choiceList.get(selection);
            api.addToFleet(side, flagship, FleetMemberType.SHIP, true);
            choiceList.remove(selection);
                
            int i = 0;
            while (i < numOpponents) { 
                selection = (int) (Math.random() * (float) choiceList.size());
                String aiship = choiceList.get(selection);
                api.addToFleet(FleetSide.PLAYER, aiship, FleetMemberType.SHIP, false).getCaptain().setPersonality(Personalities.RECKLESS);
                choiceList.remove(selection);
                i++;
            }
	}

	public void defineMission(MissionDefinitionAPI api) {

		// Set up the fleets so we can add ships and fighter wings to them.
		// In this scenario, the fleets are attacking each other, but
		// in other scenarios, a fleet may be defending or trying to escape
		api.initFleet(FleetSide.PLAYER, "", FleetGoal.ESCAPE, false, -5);
		api.initFleet(FleetSide.ENEMY, "", FleetGoal.ATTACK, true);

		// Set a small blurb for each fleet that shows up on the mission detail and
		// mission results screens to identify each side.
		api.setFleetTagline(FleetSide.PLAYER, "Your ship and competitors");
		api.setFleetTagline(FleetSide.ENEMY, "Course marker");
		
		// These show up as items in the bulleted list under 
		// "Tactical Objectives" on the mission detail screen
                api.addBriefingItem("Reach the edge of the asteroid field as fast as you can");
                //api.addBriefingItem("Fly THROUGH the gateway to stop your timer and finish the stage");
                api.addBriefingItem("Reactivate your system with [F] to kill the boost if you suddenly need to maneuver");
                api.addBriefingItem("Fast-moving asteroids will knock you off course - try to avoid collisions");
		
		// Friendly ships
                generateFleet(FleetSide.PLAYER, ships, api);

		// Enemy forces
		api.addToFleet(FleetSide.ENEMY, "dara_slalomflag_green_Hull", FleetMemberType.SHIP, "LEFT", true);
                api.addToFleet(FleetSide.ENEMY, "dara_slalomflag_green_Hull", FleetMemberType.SHIP, "RIGHT", false);
                //api.addToFleet(FleetSide.ENEMY, "dara_race_finishline_Standard", FleetMemberType.SHIP, true);
		
		// Set up the map.
		float width = 6000f;
		float height = 26000f;
		api.initMap((float)-width/2f, (float)width/2f, (float)-height/2f, (float)height/2f);
		
		float minX = -width/2;
		float minY = -height/2;
		
		// Add an asteroid field and a debris field
		api.addAsteroidField(0f, 0f, -45, 13000f, 75f, 300f, 200);
                api.addAsteroidField(0f, 0f, -135, 13000f, 75f, 300f, 200);
                api.addRingAsteroids(0f, 0f, -90, 15000f, 150f, 500f, 150);
		
		//api.addPlanet(0, 0, 50f, "star_yellow", 250f, true);
		
	}

}
