package data.scripts.weapons.ai;

import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipCommand;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lwjgl.util.vector.Vector2f;

// Simple, no-weaving, no fancy engine behaviors, direct approach KE missile AI designed to go fast
public class dara_JudgmentSpireAI extends dara_BaseMissile
{
    private static final float VELOCITY_DAMPING_FACTOR = 0.1f; // Lower numbers means it will snap in place more abruptly; set it to some negative number if you want it to corkscrew drunkenly
    private static final Vector2f ZERO = new Vector2f();

    public dara_JudgmentSpireAI(MissileAPI missile, ShipAPI launchingShip)
    {
        // Interface with the Base Missile AI script we are extending with this missile AI
        super(missile, launchingShip);
    }

    @Override
    public void advance(float amount)
    {
        // Don't do shit when the missile engine is fizzling, or missile is fading out
        if (missile.isFizzling() || missile.isFading())
        {
            return;
        }

        // If there is no target try to find one, if we cannot or target is not valid (phased?) fly in a straight line
        if (!acquireTarget(amount))
        {
            if (target == null)
            {
                missile.giveCommand(ShipCommand.ACCELERATE);
                return;
            }
        }

        float distance = MathUtils.getDistance(target.getLocation(), missile.getLocation());
        float guidance = 0.3f; // With no ECCM hullmod, leading accuracy is 30% of a perfect lead
        if (missile.getSource() != null)
        {
            guidance += Math.min(missile.getSource().getMutableStats().getMissileGuidance().getModifiedValue()
                    - missile.getSource().getMutableStats().getMissileGuidance().getBaseValue(), 1f) * 0.2f; // With ECCM hullmod, leading accuracy is 50% of a perfect lead
        }

        // Various math and shit, don't modify this code block
        Vector2f guidedTarget = intercept(missile.getLocation(), missile.getVelocity().length(), target.getLocation(), target.getVelocity());
        if (guidedTarget == null)
        {
            Vector2f projection = new Vector2f(target.getVelocity());
            float scalar = distance / (missile.getVelocity().length() + 1f);
            projection.scale(scalar);
            guidedTarget = Vector2f.add(target.getLocation(), projection, null);
        }
        Vector2f.sub(guidedTarget, target.getLocation(), guidedTarget);
        guidedTarget.scale(guidance);
        Vector2f.add(guidedTarget, target.getLocation(), guidedTarget);
        float angularDistance = MathUtils.getShortestRotation(missile.getFacing(), VectorUtils.getAngle(missile.getLocation(), guidedTarget));
        float absDAng = Math.abs(angularDistance);

        // Adjust aim greatly
        missile.giveCommand(angularDistance < 0 ? ShipCommand.TURN_RIGHT : ShipCommand.TURN_LEFT);

        // Always accelerate and go fast for we are a big ass KE missile!
        missile.giveCommand(ShipCommand.ACCELERATE);

        // Adjust aim subtly
        if (absDAng < 5)
        {
            float MFlightAng = VectorUtils.getAngle(ZERO, missile.getVelocity());
            float MFlightCC = MathUtils.getShortestRotation(missile.getFacing(), MFlightAng);
            if (Math.abs(MFlightCC) > 20)
            {
                missile.giveCommand(MFlightCC < 0 ? ShipCommand.STRAFE_LEFT : ShipCommand.STRAFE_RIGHT);
            }
        }

        // Dampen turning for smoothness
        if (absDAng < Math.abs(missile.getAngularVelocity()) * VELOCITY_DAMPING_FACTOR)
        {
            missile.setAngularVelocity(angularDistance / VELOCITY_DAMPING_FACTOR);
        }
    }
}
